package net.sf.aislib.goodies.rememberme;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.acegisecurity.Authentication;
import org.acegisecurity.providers.anonymous.AnonymousAuthenticationToken;
import org.acegisecurity.ui.logout.LogoutHandler;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.StringUtils;

public class RememberMeLogoutHandler implements LogoutHandler {

  private RememberMeDao dao;
  private String cookieName;

  /**
   * @see org.acegisecurity.ui.logout.LogoutHandler#logout(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, org.acegisecurity.Authentication)
   */
  public void logout(HttpServletRequest request, HttpServletResponse response, Authentication authentication) {
    // sanity check
    if (authentication == null) {
      return;
    }
    if (authentication instanceof AnonymousAuthenticationToken) {
      return;
    }

    Cookie[] cookies = request.getCookies();

    if ((cookies == null) || (cookies.length == 0)) {
      return ;
    }
    for (int i = 0; i < cookies.length; i++) {
      if (cookieName.equals(cookies[i].getName())) {
        String cookieValue = cookies[i].getValue();
        if (Base64.isArrayByteBase64(cookieValue.getBytes())) {
          String cookieAsPlainText = new String(Base64.decodeBase64(cookieValue.getBytes()));
          String[] cookieTokens = StringUtils.delimitedListToStringArray(cookieAsPlainText, ":");

          if (cookieTokens.length == 3) {
            Long tokenId = null;
            // try to read token id first, if it will fail, cancel cookie, and return
            try {
              tokenId = new Long(cookieTokens[0]);
            } catch (NumberFormatException ex) {
              return;
            }

            AuthenticationToken token = dao.getAuthenticationToken(tokenId);
            if (token == null) {
              return;
            }
            // Check if username is the same
            if (cookieTokens[1].equals(token.getUsername())) {
              dao.removeAuthenticationToken(tokenId);
            }
          }
        }
      }
    }
  }

  @Required
  public void setCookieName(String cookieName) {
    this.cookieName = cookieName;
  }

  /**
   * @param dao the dao to set
   */
  @Required
  public void setDao(RememberMeDao dao) {
    this.dao = dao;
  }

}
